# -*- coding: utf-8 -*-
import pytest

import pykakasi
from pykakasi.exceptions import (InvalidFlagValueException,
                                 InvalidModeValueException,
                                 UnsupportedRomanRulesException)


def test_kakasi_unknown_rule():
    with pytest.raises(UnsupportedRomanRulesException):
        kakasi = pykakasi.kakasi()
        kakasi.setMode("H", "a")
        kakasi.setMode("K", "a")
        kakasi.setMode("J", "a")
        kakasi.setMode("r", "hogefuga")


def test_kakasi_unknown_mode():
    with pytest.raises(InvalidModeValueException):
        kakasi = pykakasi.kakasi()
        kakasi.setMode("H", "a")
        kakasi.setMode("K", "a")
        kakasi.setMode("J", "X")


def test_kakasi_invalid_flag_value():
    with pytest.raises(InvalidFlagValueException):
        kakasi = pykakasi.kakasi()
        kakasi.setMode("H", "a")
        kakasi.setMode("K", "a")
        kakasi.setMode("s", "yes")



import os
import sys

import pytest


@pytest.fixture(scope="session", autouse=True)
def dictionary_setup_fixture():
    root_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    if "TOX_ENV_DIR" not in os.environ:
        sys.path.insert(1, os.path.join(root_dir, 'src'))
        import kakasidict
        from pykakasi.properties import Configurations
        dpath = os.path.join(root_dir, 'build', 'lib', 'pykakasi', 'data')
        print("Generating kanwa dictionary in %s\n" % dpath)
        kanwa = kakasidict.Genkanwadict()
        kanwa.generate_dictionaries(dpath)
        Configurations().data_path = dpath



# -*- coding: utf-8 -*-

import pykakasi


def test_kakasi_structured_constitution():

    original_text = "日本国民は、正当に選挙された国会における代表者を通じて行動し、われらとわれらの子孫のために、" \
                    "諸国民との協和による成果と、わが国全土にわたつて自由のもたらす恵沢を確保し、" \
                    "政府の行為によつて再び戦争の惨禍が起ることのないやうにすることを決意し、ここに主権が国民に存することを宣言し、" \
                    "この憲法を確定する。そもそも国政は、国民の厳粛な信託によるものであつて、その権威は国民に由来し、" \
                    "その権力は国民の代表者がこれを行使し、その福利は国民がこれを享受する。これは人類普遍の原理であり、" \
                    "この憲法は、かかる原理に基くものである。われらは、これに反する一切の憲法、法令及び詔勅を排除する。"

    expected = [{'orig': "日本国民", 'kana': "ニホンコクミン", 'hira': "にほんこくみん", 'hepburn': "nihonkokumin", 'kunrei': "nihonkokumin", 'passport': "nihonkokumin"},
                {'orig': "は、", 'kana': "ハ、", 'hira': "は、", 'hepburn': "ha,", 'kunrei': "ha,", 'passport': "ha,"},
                {'orig': "正当", 'kana': "セイトウ", 'hira': "せいとう", 'hepburn': "seitou", 'kunrei': "seitou", 'passport': "seito"},
                {'orig': "に", 'kana': "ニ", 'hira': "に", 'hepburn': "ni", 'kunrei': "ni", 'passport': "ni"},
                {'orig': "選挙", 'kana': "センキョ", 'hira': "せんきょ", 'hepburn': "senkyo", 'kunrei': "senkyo", 'passport': "senkyo"},
                {'orig': "された", 'kana': "サレタ", 'hira': "された", 'hepburn': "sareta", 'kunrei': "sareta", 'passport': "sareta"},
                {'orig': "国会", 'kana': "コッカイ", 'hira': "こっかい", 'hepburn': "kokkai", 'kunrei': "kokkai", 'passport': "kokkai"},
                {'orig': "における", 'kana': "ニオケル", 'hira': "における", 'hepburn': "niokeru", 'kunrei': "niokeru", 'passport': "niokeru"},
                {'orig': "代表者", 'kana': "ダイヒョウシャ", 'hira': "だいひょうしゃ", 'hepburn': "daihyousha", 'kunrei': "daihyousya", 'passport': "daihyousha"},
                {'orig': "を", 'kana': "ヲ", 'hira': "を", 'hepburn': "wo", 'kunrei': "wo", 'passport': "wo"},
                {'orig': "通じ", 'kana': "ツウジ", 'hira': "つうじ", 'hepburn': "tsuuji", 'kunrei': "tuuzi", 'passport': "tsuuji"},
                {'orig': "て", 'kana': "テ", 'hira': "て", 'hepburn': "te", 'kunrei': "te", 'passport': "te"},
                {'orig': "行動", 'kana': "コウドウ", 'hira': "こうどう", 'hepburn': "koudou", 'kunrei': "koudou", 'passport': "kodou"},
                {'orig': "し、", 'kana': "シ、", 'hira': "し、", 'hepburn': 'shi,', 'kunrei': "si,", 'passport': "shi,"},
                {'orig': "われらとわれらの", 'kana': "ワレラトワレラノ", 'hira': "われらとわれらの",
                 'hepburn': "wareratowarerano", 'kunrei': "wareratowarerano", 'passport': "wareratowarerano"},
                {'orig': "子孫", 'kana': "シソン", 'hira': "しそん", 'hepburn': "shison", 'kunrei': "sison", 'passport': "shison"},
                {'orig': "のために、", 'kana': "ノタメニ、", 'hira': "のために、", 'hepburn': "notameni,", 'kunrei': "notameni,", 'passport': "notameni,"},
                {'orig': "諸国民", 'kana': "ショコクミン", 'hira': "しょこくみん", 'hepburn': "shokokumin", 'kunrei': "syokokumin", 'passport': "shokokumin"},
                {'orig': "との", 'kana': "トノ", 'hira': "との", 'hepburn': "tono", 'kunrei': "tono", 'passport': "tono"},
                {'orig': "協和", 'kana': "キョウワ", 'hira': "きょうわ", 'hepburn': "kyouwa", 'kunrei': "kyouwa", 'passport': "kyouwa"},
                {'orig': "による", 'kana': "ニヨル", 'hira': "による", 'hepburn': "niyoru", 'kunrei': "niyoru", 'passport': "niyoru"},
                {'orig': "成果", 'kana': "セイカ", 'hira': "せいか", 'hepburn': "seika", 'kunrei': "seika", 'passport': "seika"},
                {'orig': "と、", 'kana': "ト、", 'hira': "と、", 'hepburn': "to,", 'kunrei': "to,", 'passport': "to,"},
                {'orig': "わが", 'kana': "ワガ", 'hira': "わが", 'hepburn': "waga", 'kunrei': "waga", 'passport': "waga"},
                {'orig': "国", 'kana': "クニ", 'hira': "くに", 'hepburn': "kuni", 'kunrei': "kuni", 'passport': "kuni"},
                {'orig': "全土", 'kana': "ゼンド", 'hira': "ぜんど", 'hepburn': "zendo", 'kunrei': "zendo", 'passport': "zendo"},
                {'orig': "にわたつて", 'kana': "ニワタツテ", 'hira': "にわたつて", 'hepburn': "niwatatsute", 'kunrei': "niwatatute", 'passport': "niwatatsute"},
                {'orig': "自由", 'kana': "ジユウ", 'hira': "じゆう", 'hepburn': "jiyuu", 'kunrei': "ziyuu", 'passport': "jiyuu"},
                {'orig': "のもたらす", 'kana': "ノモタラス", 'hira': 'のもたらす', 'hepburn': "nomotarasu", 'kunrei': "nomotarasu", 'passport': "nomotarasu"},
                {'orig': "恵沢", 'kana': "ケイタク", 'hira': "けいたく", 'hepburn': "keitaku", 'kunrei': "keitaku", 'passport': "keitaku"},
                {'orig': "を", 'kana': "ヲ", 'hira': "を", 'hepburn': 'wo', 'kunrei': 'wo', 'passport': 'wo'},
                {'orig': "確保", 'kana': "カクホ", 'hira': "かくほ", 'hepburn': "kakuho", 'kunrei': "kakuho", 'passport': "kakuho"},
                {'orig': "し、", 'kana': "シ、", 'hira': "し、", 'hepburn': "shi,", 'kunrei': "si,", 'passport': "shi,"},
                {'orig': "政府", 'kana': "セイフ", 'hira': "せいふ", 'hepburn': "seifu", 'kunrei': "seifu", 'passport': "seifu"},
                {'orig': "の", 'kana': "ノ", 'hira': "の", 'hepburn': "no", 'kunrei': "no", 'passport': "no"},
                {'orig': "行為", 'kana': "コウイ", 'hira': "こうい", 'hepburn': "koui", 'kunrei': "koui", 'passport': "koi"},
                {'orig': "によつて", 'kana': "ニヨツテ", 'hira': "によつて", 'hepburn': "niyotsute", 'kunrei': "niyotute", 'passport': "niyotsute"},
                {'orig': "再び", 'kana': "フタタビ", 'hira': "ふたたび", 'hepburn': "futatabi", 'kunrei': "futatabi", 'passport': "futatabi"},
                {'orig': "戦争", 'kana': "センソウ", 'hira': "せんそう", 'hepburn': "sensou", 'kunrei': "sensou", 'passport': "senso"},
                {'orig': "の", 'kana': "ノ", 'hira': "の", 'hepburn': "no", 'kunrei': "no", 'passport': "no"},
                ]

    kakasi = pykakasi.kakasi()
    result = kakasi.convert(original_text)
    for i, e in enumerate(expected):
        assert result[i]['orig'] == e['orig']
        assert result[i]['hira'] == e['hira']
        assert result[i]['kana'] == e['kana']
        assert result[i]['hepburn'] == e['hepburn']
        assert result[i]['kunrei'] == e['kunrei']
        assert result[i]['passport'] == e['passport']



# -*- coding: utf-8 -*-
import pytest

import pykakasi


@pytest.mark.parametrize("case, expected", [
    ("", [{'orig': '', 'hira': '', 'kana': '', 'kunrei': '', 'hepburn': '', 'passport': ''}]),
    ("構成",
     [
         {'orig': "構成", 'kana': "コウセイ", 'hira': "こうせい",
          'hepburn': 'kousei', 'kunrei': "kousei", 'passport': "kosei"}
     ]),
    ('好き',
     [{'orig': '好き', 'hira': 'すき', 'kana': 'スキ', 'hepburn': 'suki', 'kunrei': 'suki', 'passport': 'suki'}]),
    ('大きい',
     [{'orig': '大きい', 'hira': 'おおきい', 'kana': 'オオキイ', 'hepburn': 'ookii', 'kunrei': 'ookii', 'passport': 'okii'}]),
    ('かんたん',
     [{'orig': 'かんたん', 'hira': 'かんたん', 'kana': 'カンタン', 'hepburn': 'kantan', 'kunrei': 'kantan', 'passport': 'kantan'}]),
    ('にゃ',
     [{'orig': 'にゃ', 'hira': 'にゃ', 'kana': 'ニャ', 'hepburn': 'nya', 'kunrei': 'nya', 'passport': 'nya'}]),
    ('っき',
     [{'orig': 'っき', 'hira': 'っき', 'kana': 'ッキ', 'hepburn': 'kki', 'kunrei': 'kki', 'passport': 'kki'}]),
    ('っふぁ',
     [{'orig': 'っふぁ', 'hira': 'っふぁ', 'kana': 'ッファ', 'hepburn': 'ffa', 'kunrei': 'ffa', 'passport': 'ffa'}]),
    ('キャ',
     [{'orig': 'キャ', 'hira': 'きゃ', 'kana': 'キャ', 'hepburn': 'kya', 'kunrei': 'kya', 'passport': 'kya'}]),
    ('キュ',
     [{'orig': 'キュ', 'hira': 'きゅ', 'kana': 'キュ', 'hepburn': 'kyu', 'kunrei': 'kyu', 'passport': 'kyu'}]),
    ('キョ',
     [{'orig': 'キョ', 'hira': 'きょ', 'kana': 'キョ', 'hepburn': 'kyo', 'kunrei': 'kyo', 'passport': 'kyo'}]),
    ('漢字とひらがな交じり文',
     [
         {'orig': '漢字', 'hira': 'かんじ', 'kana': 'カンジ', 'hepburn': 'kanji', 'kunrei': 'kanzi', 'passport': 'kanji'},
         {'orig': 'とひらがな', 'hira': 'とひらがな', 'kana': 'トヒラガナ', 'hepburn': 'tohiragana', 'kunrei': 'tohiragana', 'passport': 'tohiragana'},
         {'orig': '交じり', 'hira': 'まじり', 'kana': 'マジリ', 'hepburn': 'majiri', 'kunrei': 'maziri', 'passport': 'majiri'},
         {'orig': '文', 'hira': 'ぶん', 'kana': 'ブン', 'hepburn': 'bun', 'kunrei': 'bun', 'passport': 'bun'}
     ]),
    ('Alphabet 123 and 漢字',
     [
         {'orig': 'Alphabet 123 and ', 'hira': 'Alphabet 123 and ', 'kana': 'Alphabet 123 and ',
             'hepburn': 'Alphabet 123 and ', 'kunrei': 'Alphabet 123 and ', 'passport': 'Alphabet 123 and '},
         {'orig': '漢字', 'hira': 'かんじ', 'kana': 'カンジ', 'hepburn': 'kanji', 'kunrei': 'kanzi', 'passport': 'kanji'}
     ]),
    ('日経新聞',
     [{'orig': '日経新聞', 'hira': 'にっけいしんぶん', 'kana': 'ニッケイシンブン', 'hepburn': 'nikkeishinbun', 'kunrei': 'nikkeisinbun', 'passport': 'nikkeishimbun'}]),
    ('日本国民は、',
     [
         {'orig': '日本国民', 'hira': 'にほんこくみん', 'kana': 'ニホンコクミン', 'hepburn': 'nihonkokumin', 'kunrei': 'nihonkokumin', 'passport': 'nihonkokumin'},
         {'orig': 'は、', 'hira': 'は、', 'kana': 'ハ、', 'hepburn': 'ha,', 'kunrei': 'ha,', 'passport': 'ha,'}
     ]),
    ("私がこの子を助けなきゃいけないってことだよね",
     [
         {'orig': "私", 'kana': "ワタシ", 'hira': "わたし", 'hepburn': "watashi", 'kunrei': "watasi", 'passport': "watashi"},
         {'orig': "がこの", 'kana': "ガコノ", 'hira': "がこの", 'hepburn': "gakono", 'kunrei': "gakono", 'passport': "gakono"},
         {'orig': "子", 'kana': "コ", 'hira': "こ", 'hepburn': "ko", 'kunrei': "ko", 'passport': "ko"},
         {'orig': "を", 'kana': "ヲ", 'hira': "を", 'hepburn': "wo", 'kunrei': "wo", 'passport': "wo"},
         {'orig': "助け", 'kana': "タスケ", 'hira': "たすけ", 'hepburn': "tasuke", 'kunrei': "tasuke", 'passport': "tasuke"},
         {'orig': "なきゃいけないってことだよね", 'kana': "ナキャイケナイッテコトダヨネ", 'hira': "なきゃいけないってことだよね",
          'hepburn': "nakyaikenaittekotodayone", 'kunrei': "nakyaikenaittekotodayone",
          'passport': "nakyaikenaittekotodayone"}
     ]),
    ('やったー',
     [{'orig': 'やったー', 'kana': "ヤッター", 'hira': "やったー", 'hepburn': 'yattaa', 'kunrei': 'yattaa', 'passport': "yattaa"}]),
    ('でっでー',
     [{'orig': 'でっでー', 'kana': "デッデー", 'hira': 'でっでー', 'hepburn': 'deddee', 'kunrei': 'deddee', 'passport': 'deddee'}]),
    ('てんさーふろー', [{
        'orig': 'てんさーふろー', 'kana': "テンサーフロー", 'hira': 'てんさーふろー',
        'hepburn': 'tensaafuroo', 'kunrei': 'tensaafuroo', 'passport': 'tensaafuroo'}]),
])
def test_kakasi_structured(case, expected):
    kakasi = pykakasi.kakasi()
    result = kakasi.convert(case)
    assert len(result) == len(expected)
    for i, r in enumerate(result):
        assert r['orig'] == expected[i]['orig']
        assert r['hira'] == expected[i]['hira']
        assert r['kana'] == expected[i]['kana']
        assert r['hepburn'] == expected[i]['hepburn']
        assert r['kunrei'] == expected[i]['kunrei']
        assert r['passport'] == expected[i]['passport']



#!/usr/bin/env python
import os
import sys

from setuptools import setup
from setuptools.command.build_py import build_py


class MyBuild(build_py):

    def run(self):
        build_py.run(self)

        root_dir = os.path.abspath(os.path.dirname(__file__))
        sys.path.insert(1, os.path.join(root_dir, 'src'))

        from kakasidict import Genkanwadict
        if not self.dry_run:
            kanwa = Genkanwadict()
            dstdir = os.path.join(self.build_lib, 'pykakasi', 'data')
            kanwa.generate_dictionaries(dstdir)


setup(cmdclass={'build_py': MyBuild}, use_scm_version={"local_scheme": "no-local-version"})



import codecs
import os
import re
from typing import Dict, List, Tuple

from klepto.archives import file_archive  # type: ignore # noqa

root_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


class Genkanwadict:
    records = {}  # type: Dict[str, Dict[str, List[Tuple[str, str]]]]

    ESCAPE_SEQUENCE_RE = re.compile(r'''
        ( \\U........      # 8-digit hex escapes
        | \\u....          # 4-digit hex escapes
        | \\x..            # 2-digit hex escapes
        | \\[0-7]{1,3}     # Octal escapes
        | \\N\{[^}]+\}     # Unicode characters by name
        | \\[\\'"abfnrtv]  # Single-character escapes
        )''', re.UNICODE | re.VERBOSE)

    def decode_escapes(self, s: str) -> str:
        def decode_match(match):
            return codecs.decode(match.group(0), 'unicode-escape')
        return self.ESCAPE_SEQUENCE_RE.sub(decode_match, s)

    def run(self, src: str, dst: str):
        with open(src, 'r', encoding="utf-8") as f:
            for line in f:
                self.parsekdict(line.strip())
            f.close()
        self.kanwaout(dst)

    # for kana dict

    def mkdict(self, src: str, dst: str):
        max_key_len = 0
        dic = {}
        with open(src, "r", encoding="utf-8") as f:
            for raw in f:
                line = raw.strip()
                if line.startswith(';;'):  # skip comment
                    continue
                if re.match(r"^$", line):
                    continue
                try:
                    (v, k) = self.decode_escapes(line).split(' ')
                    dic[k] = v
                    max_key_len = max(max_key_len, len(k))
                except ValueError:
                    raise Exception("Cannot process dictionary line: ", line)
        d = file_archive(dst, dic, serialized=True)
        d['_max_key_len_'] = max_key_len
        d.dump()

    def maketrans(self, src, dst):
        dict = {}
        with open(src, 'r', encoding='utf-8') as f:
            for raw in f:
                line = raw.strip()
                if line.startswith(';;'):  # skip commnet
                    continue
                if re.match(r"^$", line):
                    continue
                try:
                    (v, k) = self.decode_escapes(line).split(' ')
                    dict[ord(k)] = v
                except ValueError:
                    raise Exception("Cannot process dictionary line: ", line)
        for i in range(0xFE00, 0xFE02):
            dict[i] = None
        for i in range(0xE0100, 0xE01EF):
            dict[i] = None
        d = file_archive(dst, dict, serialized=True)
        d.dump()

    # for kanwadict

    def parsekdict(self, line: str):
        if line.startswith(';;'):  # skip comment
            return
        (yomi, kanji) = line.split(' ')
        if ord(yomi[-1:]) <= ord('z'):
            tail = yomi[-1:]
            yomi = yomi[:-1]
        else:
            tail = ''
        self.updaterec(kanji, yomi, tail)

    def updaterec(self, kanji: str, yomi: str, tail: str):
        key = "%04x" % ord(kanji[0])
        if key in self.records:
            if kanji in self.records[key]:
                rec = self.records[key][kanji]
                rec.append((yomi, tail))
                self.records[key].update({kanji: rec})
            else:
                self.records[key][kanji] = [(yomi, tail)]
        else:
            self.records[key] = {}
            self.records[key][kanji] = [(yomi, tail)]

    def kanwaout(self, out):
        dic = file_archive(out, self.records, serialized=True)
        dic.dump()

    def generate_dictionaries(self, dstdir):
        DICTS = [
            ('hepburndict.utf8', 'hepburndict3.db'),
            ('kunreidict.utf8', 'kunreidict3.db'),
            ('passportdict.utf8', 'passportdict3.db'),
            ('hepburnhira.utf8', 'hepburnhira3.db'),
            ('kunreihira.utf8', 'kunreihira3.db'),
            ('passporthira.utf8', 'passporthira3.db'),
            ('halfkana.utf8', 'halfkana3.db')
        ]
        srcdir = os.path.join(root_dir, 'src', 'data')
        if not os.path.exists(dstdir):
            os.makedirs(dstdir)
        for (src_f, pkl_f) in DICTS:
            src = os.path.join(srcdir, src_f)
            dst = os.path.join(dstdir, pkl_f)
            if (os.path.exists(dst)):
                os.unlink(dst)
            self.mkdict(src, dst)

        src = os.path.join(srcdir, 'itaijidict.utf8')
        dst = os.path.join(dstdir, 'itaijidict4.db')
        if (os.path.exists(dst)):
            os.unlink(dst)
        self.maketrans(src, dst)

        src = os.path.join(srcdir, 'kakasidict.utf8')
        dst = os.path.join(dstdir, 'kanwadict4.db')
        if (os.path.exists(dst)):
            os.unlink(dst)
        self.run(src, dst)



# -*- coding: utf-8 -*-
import pytest

import pykakasi


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("構成", "Kousei"),
    ("好き", "Suki"),
    ("大きい", "Ookii"),
    ("かんたん", "kantan"),
    ("にゃ", "nya"),
    ("っき", "kki"),
    ("っふぁ", "ffa"),
    ("キャ", "kya"),
    ("キュ", "kyu"),
    ("キョ", "kyo"),
    ("漢字とひらがな交じり文", "Kanji tohiragana Majiri Bun"),
    ("Alphabet 123 and 漢字", "Alphabet 123 and Kanji"),
    ("日経新聞", "Nikkeishinbun"),
    ("日本国民は、", "Nihonkokumin ha,")
])
def test_kakasi_hepburn(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Hepburn")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", True)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("構成", "Kousei"),
    ("好き", "Suki"),
    ("大きい", "Ookii"),
    ("かんたん", "kantan"),
    ("にゃ", "nya"),
    ("っき", "kki"),
    ("っふぁ", "ffa"),
    ("漢字とひらがな交じり文", "Kanzi tohiragana Maziri Bun"),
    ("Alphabet 123 and 漢字", "Alphabet 123 and Kanzi"),
    ("日経新聞", "Nikkeisinbun"),
    ("日本国民は、", "Nihonkokumin ha,")
])
def test_kakasi_kunrei(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Kunrei")
    kakasi.setMode("E", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("a", None)
    kakasi.setMode("C", True)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("構成", "こうせい"),
    ("好き", "すき"),
    ("大きい", "おおきい"),
    ("かんたん", "かんたん"),
    ("にゃ", "にゃ"),
    ("っき", "っき"),
    ("っふぁ", "っふぁ"),
    ("漢字とひらがな交じり文", "かんじとひらがなまじりぶん"),
    ("Alphabet 123 and 漢字", "Alphabet 123 and かんじ"),
    ("日経新聞", "にっけいしんぶん"),
    ("日本国民は、", "にほんこくみんは、"),
    ("苦々しい", "にがにがしい")
])
def test_kakasi_J2H(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "H")
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("構成", "コウセイ"),
    ("好き", "スキ"),
    ("大きい", "オオキイ"),
    ("かんたん", "かんたん"),
    ("漢字とひらがな交じり文", "カンジとひらがなマジリブン"),
    ("Alphabet 123 and 漢字", "Alphabet 123 and カンジ")
])
def test_kakasi_J2K(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "K")
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("かんたん", "カンタン"),
    ("にゃ", "ニャ")
])
def test_kakasi_H2K(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "K")
    kakasi.setMode("S", " ")
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("カンタン", "かんたん"),
    ("ニャ", "にゃ")
])
def test_kakasi_K2H(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("K", "H")
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("交じり文", "交じり 文"),
    ("ひらがな交じり文", "ひらがな 交じり 文"),
    ("漢字とひらがな交じり文", "漢字 とひらがな 交じり 文")
])
def test_wakati(case, expected):
    wakati = pykakasi.wakati()
    converter = wakati.getConverter()
    assert converter.do(case) == expected


def test_katakana_furiagana():
    TESTS = [
        (u"変換前の漢字の脇に", u"変換前[ヘンカンマエ]の漢字[カンジ]の脇[ワキ]に")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "KF")
    kakasi.setMode("f", True)
    kakasi.setMode("s", False)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_hiragana_furiagana():
    TESTS = [
        (u"変換前の漢字の脇に", u"変換前[へんかんまえ]の漢字[かんじ]の脇[わき]に")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "HF")
    kakasi.setMode("f", True)
    kakasi.setMode("s", False)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


@pytest.mark.xfail(reason="Not implemented yet furigana mode for wakati")
def test_wakati_furiagana():
    TESTS = [
        (u"変換前の漢字の脇に", u"変換前[へんかんまえ] の 漢字[かんじ] の 脇[わき] に")
    ]
    kakasi = pykakasi.wakati()
    kakasi.setMode("f", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


@pytest.mark.parametrize("case, expected", [
    ("ABCDEFGHIJKLMNOPQRSTUVWXYZ", u"ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ"),
    ("abcdefghijklmnopqrstuvwxyz", u"ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ"),
    ("!\"#$%&'()*+,-./_ {|}~", u"！＂＃＄％＆＇（）＊＋，－．／＿　｛｜｝～")
])
def test_kakasi_a2E(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("a", "E")
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"),
    ("ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ", "abcdefghijklmnopqrstuvwxyz"),
    ("！＂＃＄％＆＇（）＊＋，－．／＿　｛｜｝～\uFF1A", "!\"#$%&'()*+,-./_ {|}~:")
])
def test_kakasi_E2a(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("E", "a")
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


def test_kakasi_E2a_upper():
    TESTS = [
        (u"ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"),
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("E", "a")
    kakasi.setMode("U", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_J2a_upper():
    TESTS = [
        (u"かな漢字", "kana KANJI"),
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("J", "a")
    kakasi.setMode("H", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("U", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_numbers():

    TESTS = [
        (u"１２３４５６７８９０", "1234567890"),
        (u"一 二 三 四 五 六 七 八 九 〇", "ichi ni san shi go roku shichi hachi kyuu (maru)")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("E", "a")
    kakasi.setMode("J", "a")
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("構成", "Kosei"),
    ("大野", "Ono"),
    ("斎藤", "Saito"),
    ("菅野", "Kanno"),
    ("本田", "Honda"),
    ("一式", "Isshiki"),
    ("別府", "Beppu"),
    ("ジェ", "jie"),
    ("チェ", "chie"),
    ("ティ", "tei"),
    ("ディ", "dei"),
    ("デュ", "deyu"),
    ("ファ", "fua"),
    ("フィ", "fui"),
    ("フェ", "fue"),
    ("フォ", "fuo"),
    ("ヴァ", "bua"),
    ("ヴィ", "bui"),
    ("ヴ", "bu"),
    ("ヴェ", "bue"),
    ("ヴォ", "buo"),
    ("キャ", "kya"),
    ("キュ", "kyu"),
    ("キョ", "kyo"),
    ("じぇ", "jie"),
    ("ちぇ", "chie"),
    ("てぃ", "tei"),
    ("でぃ", "dei"),
    ("でゅ", "deyu"),
    ("ふぁ", "fua"),
    ("ふぃ", "fui"),
    ("ふぇ", "fue"),
    ("ふぉ", "fuo")
])
def test_kakasi_passport(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Passport")
    kakasi.setMode("E", "a")
    kakasi.setMode("C", True)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("えっちゅう", "etchu"),
    ("はっちょう", "hatcho"),
    ("エッチュウ", "etchu"),
    ("ハッチョウ", "hatcho")
])
def test_kakasi_passport_specialcase(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Passport")
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("", ""),
    ("構成", "kousei"),
    ("好き", "suki"),
    ("大きい", "ookii"),
    ("かんたん", "kantan"),
    ("にゃ", "nya"),
    ("っき", "kki"),
    ("っふぁ", "ffa"),
    ("漢字とひらがな交じり文", "kanji tohiragana majiri bun"),
    ("Alphabet 123 and 漢字", "Alphabet 123 and kanji"),
    ("日経新聞", "nikkeishinbun"),
    ("日本国民は、", "nihonkokumin ha,")
])
def test_kakasi_hepburn_nocapital(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Hepburn")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ("\U0001b150", "wi"),
    ("\U0001b151", "we")
])
def test_kakasi_extended_kana(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Hepburn")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


def test_kakasi_chinese_kanji():
    TESTS = [
        (u"您好", u'您 kou')
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_chinese_kanji_replace():
    TESTS = [
        (u"您好", u'??? kou')
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    kakasi.setMode("t", False)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


@pytest.mark.parametrize("case, expected", [
    ('やったー', 'yattaa'),
    ('でっでー', 'deddee'),
    ('てんさーふろー', 'tensaafuroo'),
    ('がっがーん', 'gaggaan'),
    ('どーん', 'doon')
])
def test_kakasi_long_symbol_H(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode('H', 'a')
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    (u'ヤッター', u'yattaa'),
    (u'デッデー', u'deddee'),
    (u'テンサーフロー', u'tensaafuroo'),
    (u'ガッガーン', u'gaggaan'),
    (u'ドーン', u'doon')
])
def test_kakasi_long_symbol_K(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ('じゃーんデデーン', 'jaandedeen'),
    ('デッデーンじゃーん', 'deddeenjaan'),
    ('テンサーふろー', 'tensaafuroo'),
    ('ガッガーン', 'gaggaan'),
    ('ドーン', 'doon')
])
def test_kakasi_long_symbol_mixed_HK(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    kakasi.setMode('H', 'a')
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


@pytest.mark.parametrize("case, expected", [
    ('順じゃーんデデーン', 'junjaandedeen'),
    ('デッデーン順じゃーん', 'deddeenjunjaan'),
    ('テンサーふろー風呂', 'tensaafuroofuro'),
    ('ガッガーン癌', 'gaggaangan'),
    ('ドーン', 'doon')
])
def test_kakasi_long_symbol_mixed_JHK(case, expected):
    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    kakasi.setMode('H', 'a')
    kakasi.setMode('J', 'a')
    converter = kakasi.getConverter()
    assert converter.do(case) == expected


def test_kakasi_long_symbol_with_no_HK():
    TESTS = [
        (u'順ーデデーン', u'jun-dedeen'),
        (u'デッデーン順ー', u'deddeenjun-')
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    kakasi.setMode('H', 'a')
    kakasi.setMode('J', 'a')
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result
